@@ -28,6 +28,7 @@
 #include "gtkrc-utils.h"
 #include "gedit-message-area.h"
 #include "wp-cellrenderer.h"
+#include "caption-cellrenderer.h"
 
 #include <glib/gi18n.h>
 #include <libwindow-settings/gnome-wm-manager.h>
@@ -40,6 +41,7 @@
 enum
 {
   RESPONSE_APPLY_BG,
+  RESPONSE_REVERT_FONT,
   RESPONSE_APPLY_FONT
 };
 
@@ -112,8 +114,6 @@ theme_thumbnail_update (GdkPixbuf *pixbuf,
       g_free (path);
     }
   }
-
-  g_object_unref (pixbuf);
 }
 
 static GdkPixbuf *
@@ -157,9 +157,10 @@ theme_thumbnail_generate (GnomeThemeMetaInfo *info, AppearanceData *data)
 
   thumb = theme_get_thumbnail_from_cache (info, data);
 
-  if (thumb != NULL)
+  if (thumb != NULL) {
     theme_thumbnail_update (thumb, info->name, data, FALSE);
-  else
+    g_object_unref (thumb);
+  } else
     generate_meta_theme_thumbnail_async (info,
         (ThemeThumbnailFunc) theme_thumbnail_done_cb, data, NULL);
 }
@@ -349,6 +350,67 @@ theme_is_equal (const GnomeThemeMetaInfo *a, const GnomeThemeMetaInfo *b)
   return TRUE;
 }
 
+static gchar *
+theme_validate (const GnomeThemeMetaInfo *info)
+{
+  GnomeThemeInfo *theme;
+  gchar *gtkrc;
+
+  theme = gnome_theme_info_find (info->gtk_theme_name);
+  if (!theme || !theme->has_gtk) {
+    return g_strdup_printf (
+        _("This theme will not look as intended because the required GTK+ theme '%s' is not installed."),
+          info->gtk_theme_name);
+  }
+
+  theme = gnome_theme_info_find (info->metacity_theme_name);
+  if (!theme || !theme->has_metacity) {
+    return g_strdup_printf (
+        _("This theme will not look as intended because the required window manager theme '%s' is not installed."),
+          info->metacity_theme_name);
+  }
+
+  if (!gnome_theme_icon_info_find (info->icon_theme_name)) {
+    return g_strdup_printf (
+        _("This theme will not look as intended because the required icon theme '%s' is not installed."),
+        info->gtk_theme_name);
+  }
+
+  /* check for gtk theme engines */
+  gtkrc = gtkrc_find_named (info->gtk_theme_name);
+  if (gtkrc) {
+    GSList *engines = NULL, *l;
+    gchar *msg = NULL;
+    gboolean found;
+
+    gtkrc_get_details (gtkrc, &engines, NULL);
+    g_free (gtkrc);
+
+    for (l = engines; l; l = l->next) {
+      gchar *lib = g_strconcat ("lib", l->data, ".so", NULL);
+      gchar *full = g_build_filename (GTK_ENGINE_DIR, lib, NULL);
+
+      g_free (lib);
+      found = g_file_test (full, G_FILE_TEST_EXISTS);
+      g_free (full);
+
+      if (!found) {
+        msg = g_strdup_printf (
+            _("This theme will not look as intended because the required GTK+ theme engine '%s' is not installed."),
+            (gchar *) l->data);
+        break;
+      }
+    }
+
+    g_slist_foreach (engines, (GFunc) g_free, NULL);
+    g_slist_free (engines);
+
+    return msg;
+  }
+
+  return NULL;
+}
+
 static void
 theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *data)
 {
@@ -416,12 +478,28 @@ theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *dat
 
 /** GUI Callbacks **/
 
+static void
+custom_font_cb (GtkWidget *button, AppearanceData *data)
+{
+  g_free (data->revert_application_font);
+  g_free (data->revert_documents_font);
+  g_free (data->revert_desktop_font);
+  g_free (data->revert_windowtitle_font);
+  g_free (data->revert_monospace_font);
+  data->revert_application_font = NULL;
+  data->revert_documents_font = NULL;
+  data->revert_desktop_font = NULL;
+  data->revert_windowtitle_font = NULL;
+  data->revert_monospace_font = NULL;
+}
+
 static void
 theme_message_area_response_cb (GtkWidget *w,
                                 gint response_id,
                                 AppearanceData *data)
 {
   const GnomeThemeMetaInfo *theme;
+  gchar *tmpfont;
 
   theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
   if (!theme)
@@ -434,18 +512,118 @@ theme_message_area_response_cb (GtkWidget *w,
                                theme->background_image, NULL);
       break;
 
+    case RESPONSE_REVERT_FONT:
+      if (data->revert_application_font != NULL) {
+        gconf_client_set_string (data->client, APPLICATION_FONT_KEY,
+                                 data->revert_application_font, NULL);
+        g_free (data->revert_application_font);
+        data->revert_application_font = NULL;
+      }
+
+      if (data->revert_documents_font != NULL) {
+        gconf_client_set_string (data->client, DOCUMENTS_FONT_KEY,
+                                 data->revert_documents_font, NULL);
+        g_free (data->revert_documents_font);
+        data->revert_documents_font = NULL;
+      }
+
+      if (data->revert_desktop_font != NULL) {
+        gconf_client_set_string (data->client, DESKTOP_FONT_KEY,
+                                 data->revert_desktop_font, NULL);
+        g_free (data->revert_desktop_font);
+        data->revert_desktop_font = NULL;
+      }
+
+      if (data->revert_windowtitle_font != NULL) {
+        gconf_client_set_string (data->client, WINDOWTITLE_FONT_KEY,
+                                 data->revert_windowtitle_font, NULL);
+        g_free (data->revert_windowtitle_font);
+        data->revert_windowtitle_font = NULL;
+      }
+
+      if (data->revert_monospace_font != NULL) {
+        gconf_client_set_string (data->client, MONOSPACE_FONT_KEY,
+                                 data->revert_monospace_font, NULL);
+        g_free (data->revert_monospace_font);
+        data->revert_monospace_font = NULL;
+      }
+      break;
+
     case RESPONSE_APPLY_FONT:
-      if (theme->application_font)
+      if (theme->application_font) {
+        tmpfont = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_application_font);
+
+          if (strcmp (theme->application_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_application_font = NULL;
+          } else
+            data->revert_application_font = tmpfont;
+        }
         gconf_client_set_string (data->client, APPLICATION_FONT_KEY,
                                  theme->application_font, NULL);
+      }
 
-      if (theme->desktop_font)
+      if (theme->documents_font) {
+        tmpfont = gconf_client_get_string (data->client, DOCUMENTS_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_documents_font);
+
+          if (strcmp (theme->documents_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_documents_font = NULL;
+          } else
+            data->revert_documents_font = tmpfont;
+        }
+        gconf_client_set_string (data->client, DOCUMENTS_FONT_KEY,
+                                 theme->documents_font, NULL);
+      }
+
+      if (theme->desktop_font) {
+        tmpfont = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_desktop_font);
+
+          if (strcmp (theme->desktop_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_desktop_font = NULL;
+          } else
+            data->revert_desktop_font = tmpfont;
+        }
         gconf_client_set_string (data->client, DESKTOP_FONT_KEY,
                                  theme->desktop_font, NULL);
+      }
 
-      if (theme->monospace_font)
+      if (theme->windowtitle_font) {
+        tmpfont = gconf_client_get_string (data->client, WINDOWTITLE_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_windowtitle_font);
+
+          if (strcmp (theme->windowtitle_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_windowtitle_font = NULL;
+          } else
+            data->revert_windowtitle_font = tmpfont;
+        }
+        gconf_client_set_string (data->client, WINDOWTITLE_FONT_KEY,
+                                 theme->windowtitle_font, NULL);
+      }
+
+      if (theme->monospace_font) {
+        tmpfont = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_monospace_font);
+
+          if (strcmp (theme->monospace_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_monospace_font = NULL;
+          } else
+            data->revert_monospace_font = tmpfont;
+        }
         gconf_client_set_string (data->client, MONOSPACE_FONT_KEY,
                                  theme->monospace_font, NULL);
+      }
       break;
   }
 }
@@ -454,8 +632,12 @@ static void
 theme_message_area_update (AppearanceData *data)
 {
   const GnomeThemeMetaInfo *theme;
-  gboolean show_apply_background = FALSE, show_apply_font = FALSE;
+  gboolean show_apply_background = FALSE;
+  gboolean show_apply_font = FALSE;
+  gboolean show_revert_font = FALSE;
+  gboolean show_error;
   const gchar *message;
+  gchar *error_message;
   gchar *font;
 
   theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
@@ -463,42 +645,64 @@ theme_message_area_update (AppearanceData *data)
   if (!theme)
     return;
 
-  if (theme->background_image != NULL) {
-    gchar *background;
+  error_message = theme_validate (theme);
+  show_error = (error_message != NULL);
 
-    background = gconf_client_get_string (data->client, BACKGROUND_KEY, NULL);
-    show_apply_background =
-        (!background || strcmp (theme->background_image, background) != 0);
-    g_free (background);
-  }
+  if (!show_error) {
+    if (theme->background_image != NULL) {
+      gchar *background;
 
-  if (theme->application_font) {
-    font = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
-  }
+      background = gconf_client_get_string (data->client, BACKGROUND_KEY, NULL);
+      show_apply_background =
+          (!background || strcmp (theme->background_image, background) != 0);
+      g_free (background);
+    }
 
-  if (!show_apply_font && theme->desktop_font) {
-    font = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
-  }
+    if (theme->application_font) {
+      font = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
 
-  if (!show_apply_font && theme->monospace_font) {
-    font = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
+    if (!show_apply_font && theme->documents_font) {
+      font = gconf_client_get_string (data->client, DOCUMENTS_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    if (!show_apply_font && theme->desktop_font) {
+      font = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    if (!show_apply_font && theme->windowtitle_font) {
+      font = gconf_client_get_string (data->client, WINDOWTITLE_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    if (!show_apply_font && theme->monospace_font) {
+      font = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    show_revert_font = (data->revert_application_font != NULL ||
+      data->revert_documents_font != NULL || data->revert_desktop_font != NULL ||
+      data->revert_windowtitle_font != NULL || data->revert_monospace_font != NULL);
   }
 
   if (data->theme_message_area == NULL) {
     GtkWidget *hbox;
-    GtkWidget *icon;
     GtkWidget *parent;
 
-    if (!show_apply_background && !show_apply_font)
+    if (!show_apply_background && !show_revert_font && !show_apply_font && !show_error)
       return;
 
     data->theme_message_area = gedit_message_area_new ();
@@ -515,6 +719,10 @@ theme_message_area_update (AppearanceData *data)
         GEDIT_MESSAGE_AREA (data->theme_message_area),
         _("Apply Font"),
         RESPONSE_APPLY_FONT);
+    data->revert_font_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->theme_message_area),
+        _("Revert Font"),
+        RESPONSE_REVERT_FONT);
 
     data->theme_message_label = gtk_label_new (NULL);
     gtk_widget_show (data->theme_message_label);
@@ -523,10 +731,12 @@ theme_message_area_update (AppearanceData *data)
 
     hbox = gtk_hbox_new (FALSE, 9);
     gtk_widget_show (hbox);
-    icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
-    gtk_widget_show (icon);
-    gtk_misc_set_alignment (GTK_MISC (icon), 0.5, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+    data->theme_info_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+    data->theme_error_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+    gtk_misc_set_alignment (GTK_MISC (data->theme_info_icon), 0.5, 0);
+    gtk_misc_set_alignment (GTK_MISC (data->theme_error_icon), 0.5, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), data->theme_info_icon, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), data->theme_error_icon, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), data->theme_message_label, TRUE, TRUE, 0);
     gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (data->theme_message_area), hbox);
 
@@ -534,27 +744,57 @@ theme_message_area_update (AppearanceData *data)
     gtk_box_pack_start (GTK_BOX (parent), data->theme_message_area, FALSE, FALSE, 0);
   }
 
-  if (show_apply_background && show_apply_font) {
-    gtk_widget_show (data->theme_message_area);
-    gtk_widget_show (data->apply_background_button);
-    gtk_widget_show (data->apply_font_button);
+  if (show_error)
+    message = error_message;
+  else if (show_apply_background && show_apply_font && show_revert_font)
+    message = _("The current theme suggests a background and a font. Also, the last applied font suggestion can be reverted.");
+  else if (show_apply_background && show_revert_font)
+    message = _("The current theme suggests a background. Also, the last applied font suggestion can be reverted.");
+  else if (show_apply_background && show_apply_font)
     message = _("The current theme suggests a background and a font.");
-  } else if (show_apply_background) {
-    gtk_widget_show (data->theme_message_area);
-    gtk_widget_show (data->apply_background_button);
-    gtk_widget_hide (data->apply_font_button);
+  else if (show_apply_font && show_revert_font)
+    message = _("The current theme suggests a font. Also, the last applied font suggestion can be reverted.");
+  else if (show_apply_background)
     message = _("The current theme suggests a background.");
-  } else if (show_apply_font) {
-    gtk_widget_show (data->theme_message_area);
-    gtk_widget_hide (data->apply_background_button);
-    gtk_widget_show (data->apply_font_button);
+  else if (show_revert_font)
+    message = _("The last applied font suggestion can be reverted.");
+  else if (show_apply_font)
     message = _("The current theme suggests a font.");
+  else
+    message = NULL;
+
+  if (show_apply_background)
+    gtk_widget_show (data->apply_background_button);
+  else
+    gtk_widget_hide (data->apply_background_button);
+
+  if (show_apply_font)
+    gtk_widget_show (data->apply_font_button);
+  else
+    gtk_widget_hide (data->apply_font_button);
+
+  if (show_revert_font)
+    gtk_widget_show (data->revert_font_button);
+  else
+    gtk_widget_hide (data->revert_font_button);
+
+  if (show_error || show_apply_background || show_apply_font || show_revert_font) {
+    gtk_widget_show (data->theme_message_area);
+    gtk_widget_queue_draw (data->theme_message_area);
+
+    if (show_error) {
+      gtk_widget_show (data->theme_error_icon);
+      gtk_widget_hide (data->theme_info_icon);
+    } else {
+      gtk_widget_show (data->theme_info_icon);
+      gtk_widget_hide (data->theme_error_icon);
+    }
   } else {
     gtk_widget_hide (data->theme_message_area);
-    message = NULL;
   }
 
   gtk_label_set_text (GTK_LABEL (data->theme_message_label), message);
+  g_free (error_message);
 }
 
 static void
@@ -819,8 +1059,15 @@ themes_init (AppearanceData *data)
   gnome_theme_init (NULL);
   gnome_wm_manager_init ();
 
+  data->revert_application_font = NULL;
+  data->revert_documents_font = NULL;
+  data->revert_desktop_font = NULL;
+  data->revert_windowtitle_font = NULL;
+  data->revert_monospace_font = NULL;
   data->theme_save_dialog = NULL;
   data->theme_message_area = NULL;
+  data->theme_info_icon = NULL;
+  data->theme_error_icon = NULL;
   data->theme_custom = gnome_theme_meta_info_new ();
   data->theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/theme-thumbnailing.png", NULL);
   data->theme_store = theme_store =
@@ -874,7 +1121,7 @@ themes_init (AppearanceData *data)
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (icon_view), renderer,
                                   "pixbuf", COL_THUMBNAIL, NULL);
 
-  renderer = gtk_cell_renderer_text_new ();
+  renderer = cell_renderer_caption_new ();
   g_object_set (renderer, "alignment", PANGO_ALIGN_CENTER,
 			  "wrap-mode", PANGO_WRAP_WORD_CHAR,
 			  "wrap-width", gtk_icon_view_get_item_width (icon_view),
@@ -928,13 +1175,28 @@ themes_init (AppearanceData *data)
 #endif
   gconf_client_notify_add (data->client, BACKGROUND_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
   gconf_client_notify_add (data->client, APPLICATION_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, DOCUMENTS_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
   gconf_client_notify_add (data->client, DESKTOP_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, WINDOWTITLE_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
   gconf_client_notify_add (data->client, MONOSPACE_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
 
   settings = gtk_settings_get_default ();
   g_signal_connect (settings, "notify::gtk-color-scheme", (GCallback) theme_setting_changed_cb, data);
   g_signal_connect (settings, "notify::gtk-theme-name", (GCallback) theme_setting_changed_cb, data);
   g_signal_connect (settings, "notify::gtk-icon-theme-name", (GCallback) theme_setting_changed_cb, data);
+
+  /* monitor individual font choice buttons, so
+     "revert font" option (if any) can be cleared */
+  w = glade_xml_get_widget (data->xml, "application_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "document_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "desktop_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "window_title_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "monospace_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
 }
 
 void
@@ -946,4 +1208,9 @@ themes_shutdown (AppearanceData *data)
     g_object_unref (data->theme_icon);
   if (data->theme_save_dialog)
     gtk_widget_destroy (data->theme_save_dialog);
+  g_free (data->revert_application_font);
+  g_free (data->revert_documents_font);
+  g_free (data->revert_desktop_font);
+  g_free (data->revert_windowtitle_font);
+  g_free (data->revert_monospace_font);
 }
