@@ -60,7 +60,15 @@ typedef struct {
   GtkWidget *theme_message_area;
   GtkWidget *theme_message_label;
   GtkWidget *apply_background_button;
+  GtkWidget *revert_font_button;
   GtkWidget *apply_font_button;
+  GtkWidget *theme_info_icon;
+  GtkWidget *theme_error_icon;
+  gchar *revert_application_font;
+  gchar *revert_documents_font;
+  gchar *revert_desktop_font;
+  gchar *revert_windowtitle_font;
+  gchar *revert_monospace_font;
 
   /* style */
   GdkPixbuf *gtk_theme_icon;
