@@ -16,7 +16,6 @@
 #include "eggcellrendererkeys.h"
 #include "activate-settings-daemon.h"
 
-#define LABEL_DATA "gnome-keybinding-properties-label"
 #define MAX_ELEMENTS_BEFORE_SCROLLING 10
 
 typedef struct {
@@ -63,28 +62,7 @@ typedef struct
   char *description;
 } KeyEntry;
 
-static void  reload_key_entries (gpointer                wm_name,
-                                 GladeXML               *dialog);
-static char* binding_name       (guint                   keyval,
-				 guint			 keycode,
-                                 EggVirtualModifierType  mask,
-                                 gboolean                translate);
-
-static GtkTreeModel*
-get_real_model (GtkTreeView *tree_view)
-{
-  GtkTreeModel *model;
-  GtkTreeModel *submodel;
-
-  model = gtk_tree_view_get_model (tree_view);
-
-  if (model)
-    submodel = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
-  else
-    submodel = NULL;
-
-  return submodel;
-}
+static gboolean block_accels = FALSE;
 
 static GladeXML *
 create_dialog (void)
@@ -120,7 +98,7 @@ binding_from_string (const char             *str,
 {
   g_return_val_if_fail (accelerator_key != NULL, FALSE);
 
-  if (str == NULL || (str && strcmp (str, "disabled") == 0))
+  if (str == NULL || strcmp (str, "disabled") == 0)
     {
       *accelerator_key = 0;
       *keycode = 0;
@@ -274,32 +252,27 @@ keyentry_sort_func (GtkTreeModel *model,
 }
 
 static void
-clear_old_model (GladeXML  *dialog)
+clear_old_model (GladeXML *dialog)
 {
   GtkWidget *tree_view;
   GtkTreeModel *model;
 
   tree_view = WID ("shortcut_treeview");
-  model = get_real_model (GTK_TREE_VIEW (tree_view));
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
 
   if (model == NULL)
     {
       /* create a new model */
-      GtkTreeModel *sort_model;
-
       model = (GtkTreeModel *) gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
 
-      sort_model = gtk_tree_model_sort_new_with_model (model);
-
-      gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model),
+      gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
                                        KEYENTRY_COLUMN,
                                        keyentry_sort_func,
                                        NULL, NULL);
 
-      gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), sort_model);
+      gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), model);
 
       g_object_unref (model);
-      g_object_unref (sort_model);
     }
   else
     {
@@ -314,8 +287,6 @@ clear_old_model (GladeXML  *dialog)
        * cached values do not have that set, though */
       gconf_client_clear_cache (client);
 
-      g_object_ref (model);
-
       for (valid = gtk_tree_model_get_iter_first (model, &iter);
            valid;
            valid = gtk_tree_model_iter_next (model, &iter))
@@ -333,8 +304,7 @@ clear_old_model (GladeXML  *dialog)
             }
         }
 
-      gtk_tree_store_clear(GTK_TREE_STORE(model));
-   	  g_object_unref (model);
+      gtk_tree_store_clear (GTK_TREE_STORE (model));
       g_object_unref (client);
     }
 
@@ -416,7 +386,7 @@ append_keys_to_tree (GladeXML           *dialog,
   gint i, j;
 
   client = gconf_client_get_default ();
-  model = get_real_model (GTK_TREE_VIEW (WID ("shortcut_treeview")));
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (WID ("shortcut_treeview")));
 
   /* Try to find a section parent iter, if it already exists */
   i = gtk_tree_model_iter_n_children (model, NULL);
@@ -717,13 +687,12 @@ append_keys_to_tree_from_file (GladeXML   *dialog,
   keys = (KeyListEntry *) g_array_free (keylist->entries, FALSE);
   if (keylist->package)
     {
+      bind_textdomain_codeset (keylist->package, "UTF-8");
       title = dgettext (keylist->package, keylist->name);
     } else {
       title = _(keylist->name);
     }
 
-  g_message ("title: %s", title);
-
   append_keys_to_tree (dialog, title, keys);
 
   g_free (keylist->name);
@@ -859,6 +828,8 @@ accel_edited_callback (GtkCellRendererText   *cell,
   GError *err = NULL;
   char *str;
 
+  block_accels = FALSE;
+
   model = gtk_tree_view_get_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_path_free (path);
@@ -873,7 +844,6 @@ accel_edited_callback (GtkCellRendererText   *cell,
   /* CapsLock isn't supported as a keybinding modifier, so keep it from confusing us */
   mask &= ~EGG_VIRTUAL_LOCK_MASK;
 
-  model = get_real_model (view);
   tmp_key.model  = model;
   tmp_key.keyval = keyval;
   tmp_key.keycode = keycode;
@@ -996,6 +966,8 @@ accel_cleared_callback (GtkCellRendererText *cell,
   GError *err = NULL;
   GtkTreeModel *model;
 
+  block_accels = FALSE;
+
   model = gtk_tree_view_get_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_path_free (path);
@@ -1072,7 +1044,7 @@ start_editing_kb_cb (GtkTreeView *treeview,
 static gboolean
 start_editing_cb (GtkTreeView    *tree_view,
 		  GdkEventButton *event,
-		  GladeXML       *dialog)
+		  gpointer        user_data)
 {
   GtkTreePath *path;
 
@@ -1098,10 +1070,26 @@ start_editing_cb (GtkTreeView    *tree_view,
       idle_data->path = path;
       g_signal_stop_emission_by_name (G_OBJECT (tree_view), "button_press_event");
       g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
+      block_accels = TRUE;
     }
   return TRUE;
 }
 
+/* this handler is used to keep accels from activating while the user
+ * is assigning a new shortcut so that he won't accidentally trigger one
+ * of the widgets */
+static gboolean
+maybe_block_accels (GtkWidget *widget,
+                    GdkEventKey *event,
+                    gpointer user_data)
+{
+  if (block_accels)
+  {
+    return gtk_window_propagate_key_event (GTK_WINDOW (widget), event);
+  }
+  return FALSE;
+}
+
 static void
 cb_dialog_response (GtkWidget *widget, gint response_id, gpointer data)
 {
@@ -1133,10 +1121,10 @@ setup_dialog (GladeXML *dialog)
 
   g_signal_connect (GTK_TREE_VIEW (WID ("shortcut_treeview")),
 		    "button_press_event",
-		    G_CALLBACK (start_editing_cb), dialog);
+		    G_CALLBACK (start_editing_cb), NULL);
   g_signal_connect (GTK_TREE_VIEW (WID ("shortcut_treeview")),
 		    "row-activated",
-		    G_CALLBACK (start_editing_kb_cb), dialog);
+		    G_CALLBACK (start_editing_kb_cb), NULL);
 
   column = gtk_tree_view_column_new_with_attributes (_("Action"),
 						     gtk_cell_renderer_text_new (),
@@ -1186,6 +1174,7 @@ setup_dialog (GladeXML *dialog)
   capplet_set_icon (widget, "gnome-settings-keybindings");
   gtk_widget_show (widget);
 
+  g_signal_connect (G_OBJECT (widget), "key_press_event", G_CALLBACK (maybe_block_accels), NULL);
   g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK (cb_dialog_response), dialog);
 }
 
